package com.usky.iot.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.iot.domain.PmProject;
import com.usky.iot.domain.PmReceive;
import com.usky.iot.domain.PmWorkContent;
import com.usky.iot.domain.PmWorkReport;
import com.usky.iot.mapper.*;
import com.usky.iot.service.PmProjectService;
import com.usky.iot.service.PmWorkContentService;
import com.usky.iot.service.vo.*;
import com.usky.system.domain.SysUser;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 工作内容表 服务实现类
 * </p>
 *
 * @author fu
 * @since 2024-05-20
 */
@Service
public class PmWorkContentServiceImpl extends AbstractCrudService<PmWorkContentMapper, PmWorkContent> implements PmWorkContentService {

    @Autowired
    private PmWorkReportMapper pmWorkReportMapper;

    @Autowired
    private SysUserMapper sysUserMapper;

    @Autowired
    private PmProjectService pmProjectService;

    @Autowired
    private PmReceiveMapper pmReceiveMapper;

    @Autowired
    private PmWorkContentMapper pmWorkContentMapper;

    @Autowired
    private PmProjectMapper pmProjectMapper;

    @Override
    public void deleteContent(Integer reportId) {
        LambdaQueryWrapper<PmWorkContent> deleteWrapper = Wrappers.lambdaQuery();
        deleteWrapper.eq(PmWorkContent::getReportId, reportId);
        baseMapper.delete(deleteWrapper);
    }

    private List<Integer> reportDateQuery(String startDate, String endDate) {
        Long deptId = SecurityUtils.getLoginUser().getSysUser().getDeptId();
        Integer tenantId = SecurityUtils.getTenantId();
        LambdaQueryWrapper<PmWorkReport> reportQuery = Wrappers.lambdaQuery();
        reportQuery.select(PmWorkReport::getId)
                .eq(PmWorkReport::getTenantId, tenantId)
                .eq(PmWorkReport::getDeptId, deptId);
        if (StringUtils.isNotBlank(startDate) && StringUtils.isNotBlank(endDate)) {
            LocalDate start = LocalDate.parse(startDate);
            LocalDate end = LocalDate.parse(endDate);
            reportQuery.between(PmWorkReport::getReportDate, start, end);
        }
        List<PmWorkReport> reportIds = pmWorkReportMapper.selectList(reportQuery);
        List<Integer> rIds = new ArrayList<>();
        for (PmWorkReport report : reportIds) {
            rIds.add(report.getId());
        }
        return rIds;
    }

    /**
     * @description: 查询我收到的报告
     * @author: fu
     * @date: 2024/8/7 17:48
     * @param: []
     * @return: java.util.List<java.lang.Integer>
     **/
    private List<Integer> receive() {
        Long userId = SecurityUtils.getUserId();
        Integer tenantId = SecurityUtils.getTenantId();
        LambdaQueryWrapper<PmWorkReport> reportQuery = Wrappers.lambdaQuery();
        reportQuery.select(PmWorkReport::getId)
                .eq(PmWorkReport::getTenantId, tenantId)
                .apply("FIND_IN_SET(" + userId + ", cc_to) > 0");
        List<PmWorkReport> reportIds = pmWorkReportMapper.selectList(reportQuery);
        return reportIds.stream().map(PmWorkReport::getId).collect(Collectors.toList());
    }

    private List<Integer> sentOut() {
        String username = SecurityUtils.getUsername();
        Integer tenantId = SecurityUtils.getTenantId();
        LambdaQueryWrapper<PmWorkReport> reportQuery = Wrappers.lambdaQuery();
        reportQuery.select(PmWorkReport::getId)
                .eq(PmWorkReport::getTenantId, tenantId)
                .eq(PmWorkReport::getCreateBy, username);
        List<PmWorkReport> reportIds = pmWorkReportMapper.selectList(reportQuery);
        List<Integer> rIds = new ArrayList<>();
        for (PmWorkReport report : reportIds) {
            rIds.add(report.getId());
        }
        return rIds;
    }

    /**
     * @description: 负责人项目id查询
     * @author: fu
     * @date: 2024/8/7 17:14
     * @param: []
     * @return: java.util.List<java.lang.Integer>
     **/
    private List<Integer> head() {
        Long userId = SecurityUtils.getUserId();
        Integer tenantId = SecurityUtils.getTenantId();
        LambdaQueryWrapper<PmProject> projectQuery = Wrappers.lambdaQuery();
        projectQuery.select(PmProject::getId)
                .eq(PmProject::getTenantId, tenantId)
                .eq(PmProject::getProjectHead, userId);
        List<PmProject> projectIds = pmProjectMapper.selectList(projectQuery);
        List<Integer> rIds = new ArrayList<>();
        for (PmProject project : projectIds) {
            rIds.add(project.getId());
        }
        return rIds;
    }

    private List<Integer> reportList(String projectName) {
        Integer tenantId = SecurityUtils.getTenantId();
        LambdaQueryWrapper<PmWorkContent> projectQuery = Wrappers.lambdaQuery();
        projectQuery.select(PmWorkContent::getReportId)
                .eq(PmWorkContent::getProjectName, projectName)
                .eq(PmWorkContent::getTenantId, tenantId);
        List<PmWorkContent> projectIds = pmWorkContentMapper.selectList(projectQuery);
        List<Integer> rIds = new ArrayList<>();
        for (PmWorkContent contenReportId : projectIds) {
            rIds.add(contenReportId.getId());
        }
        return rIds;
    }

    @Override
    public List<PmWorkContent> projectQuery(String startDate, String endDate, String projectName, Integer projectAscription, Integer reportId) {
        if (reportId != null && reportId != 0) {
            LambdaQueryWrapper<PmWorkContent> contentLambdaQuery = Wrappers.lambdaQuery();
            contentLambdaQuery.select(PmWorkContent::getProjectId, PmWorkContent::getProjectName)
                    .eq(PmWorkContent::getReportId, reportId)
                    .groupBy(PmWorkContent::getProjectId, PmWorkContent::getProjectName);
            List<PmWorkContent> pmWorkContentList = this.list(contentLambdaQuery);
            return pmWorkContentList;
        }
        List<Integer> reportIds = reportDateQuery(startDate, endDate);
        List<Integer> receiveList = receive();
        List<Integer> sentOutList = sentOut();
        List<Integer> headProjects = head();
        // LocalDate start = LocalDate.parse(startDate);
        // LocalDate end = LocalDate.parse(endDate);
        // String subQueryReports = "SELECT id FROM pm_work_report WHERE report_date BETWEEN '" + start + "' AND '" + end + "' AND tenant_id = " + SecurityUtils.getTenantId();
        // String subQueryReport = "SELECT id FROM pm_work_report WHERE FIND_IN_SET('" + SecurityUtils.getUserId() + "', cc_to) > 0 AND tenant_id = " + SecurityUtils.getTenantId();
        // String sentOut = "SELECT id FROM pm_work_report WHERE create_by = '" + SecurityUtils.getUsername() + "' AND tenant_id = " + SecurityUtils.getTenantId();
        // String subQueryProjects = "SELECT id FROM pm_project WHERE project_head = '" + SecurityUtils.getUserId() + "' AND tenant_id = " + SecurityUtils.getTenantId();
        LambdaQueryWrapper<PmWorkContent> contentLambdaQuery = Wrappers.lambdaQuery();
        contentLambdaQuery.select(PmWorkContent::getProjectId, PmWorkContent::getProjectName);
        if (!reportIds.isEmpty()) {
            contentLambdaQuery.in(PmWorkContent::getReportId, reportIds);
        }
        //.inSql(PmWorkContent::getReportId, subQueryReports);
        if (StringUtils.isNotBlank(projectName)) {
            contentLambdaQuery.like(PmWorkContent::getProjectName, projectName);
        }
        if (projectAscription == 1) {
            if (headProjects.isEmpty()) {
                return new ArrayList<PmWorkContent>();
            }
            contentLambdaQuery.in(PmWorkContent::getProjectId, headProjects);
            // contentLambdaQuery.inSql(PmWorkContent::getProjectId, subQueryProjects);
        } else if (projectAscription == 2) {
            // contentLambdaQuery.inSql(PmWorkContent::getReportId, subQueryReport);
            if (receiveList.isEmpty()) {
                return new ArrayList<PmWorkContent>();
            }
            contentLambdaQuery.in(PmWorkContent::getReportId, receiveList);
        } else if (projectAscription == 3) {
            // contentLambdaQuery.in(PmWorkContent::getReportId, sentOutList);
            if (sentOutList.isEmpty()) {
                return new ArrayList<PmWorkContent>();
            }
            contentLambdaQuery.in(PmWorkContent::getReportId, sentOutList);
        }
        contentLambdaQuery.groupBy(PmWorkContent::getProjectId, PmWorkContent::getProjectName);
        List<PmWorkContent> pmWorkContentList = this.list(contentLambdaQuery);
        return pmWorkContentList;
    }

    @Override
    public List<PmWorkReport> workReportQuery(String startDate, String endDate, String projectName, Integer projectAscription, Integer reportId) {
        List<PmWorkReport> reportList = new ArrayList<>();
        List<PmWorkContent> pmWorkContentList = new ArrayList<>();
        List<Long> userIds = new ArrayList<>();
        if (reportId != null && reportId != 0) {
            LambdaQueryWrapper<PmWorkReport> reportQuery = Wrappers.lambdaQuery();
            reportQuery.select(PmWorkReport::getId, PmWorkReport::getCoordinateWork, PmWorkReport::getTomorrowPlan, PmWorkReport::getCcTo, PmWorkReport::getSubmitterId,
                            PmWorkReport::getCreateTime, PmWorkReport::getReportDate)
                    .eq(PmWorkReport::getId, reportId);
            PmWorkReport report = pmWorkReportMapper.selectOne(reportQuery);
            LambdaQueryWrapper<PmWorkContent> contentQuery = Wrappers.lambdaQuery();
            contentQuery.select(PmWorkContent::getWorkContent, PmWorkContent::getWorkTime, PmWorkContent::getProjectName)
                    .eq(PmWorkContent::getReportId, reportId);
            pmWorkContentList = this.list(contentQuery);
            report.setWorkContents(pmWorkContentList);
            reportList.add(report);
            for (PmWorkReport c : reportList) {
                userIds.add(c.getSubmitterId());
            }
        } else {
            List<Integer> receiveList2 = receive();
            List<Integer> sentOutList = sentOut();
            List<Integer> headProjects = head();
            // String subQueryReports = "SELECT id FROM pm_work_report WHERE report_date BETWEEN '" + start + "' AND '" + end + "' AND tenant_id = " + SecurityUtils.getTenantId();
            // String subQueryReport = "SELECT id FROM pm_work_report WHERE FIND_IN_SET('" + SecurityUtils.getUserId() + "', cc_to) > 0 AND tenant_id = " + SecurityUtils.getTenantId();
            // String subQueryProjects = "SELECT id FROM pm_project WHERE project_head = '" + SecurityUtils.getUserId() + "' AND tenant_id = " + SecurityUtils.getTenantId();
            // String sentOut = "SELECT id FROM pm_work_report WHERE create_by = '" + SecurityUtils.getUsername() + "' AND tenant_id = " + SecurityUtils.getTenantId();
            // String reportIdList = "SELECT report_id FROM pm_work_content WHERE project_name = '" + projectName + "' AND tenant_id = " + SecurityUtils.getTenantId();
            LambdaQueryWrapper<PmWorkContent> contentLambdaQuery = Wrappers.lambdaQuery();
            contentLambdaQuery.select(PmWorkContent::getReportId, PmWorkContent::getWorkContent, PmWorkContent::getWorkTime, PmWorkContent::getProjectName);
            if (StringUtils.isNotBlank(startDate) && StringUtils.isNotBlank(endDate)) {
                DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
                LocalDate start = LocalDate.parse(startDate, formatter);
                LocalDate end = LocalDate.parse(endDate, formatter);
                LocalDateTime startDateTime = start.atStartOfDay();
                LocalDateTime endDateTime = end.atTime(23, 59, 59);
                contentLambdaQuery.between(PmWorkContent::getCreateTime, startDateTime, endDateTime);
            }
            if (projectAscription == 1) {
                //.inSql(PmWorkContent::getReportId, subQueryReports);
                contentLambdaQuery.in(PmWorkContent::getProjectId, headProjects);
            } else if (projectAscription == 2) {
                // contentLambdaQuery.inSql(PmWorkContent::getReportId, subQueryReport);
                contentLambdaQuery.in(PmWorkContent::getReportId, receiveList2);
            } else if (projectAscription == 3) {
                // contentLambdaQuery.inSql(PmWorkContent::getReportId, sentOut);
                contentLambdaQuery.in(PmWorkContent::getReportId, sentOutList);
            }
//            if (StringUtils.isNotBlank(projectName)) {
//                contentLambdaQuery.inSql(PmWorkContent::getReportId, reportIdList);
//            }
            if (StringUtils.isNotBlank(projectName)) {
                List<Integer> list = reportList(projectName);
                contentLambdaQuery.in(PmWorkContent::getReportId, list);
            }
            pmWorkContentList = this.list(contentLambdaQuery);
            if (pmWorkContentList.isEmpty()) {
                return reportList;
            }
            List<Integer> reportIds = new ArrayList<>();
            for (PmWorkContent a : pmWorkContentList) {
                reportIds.add(a.getReportId());
            }
            LambdaQueryWrapper<PmWorkReport> reportQuery = Wrappers.lambdaQuery();
            reportQuery.select(PmWorkReport::getId, PmWorkReport::getCoordinateWork, PmWorkReport::getTomorrowPlan, PmWorkReport::getCcTo, PmWorkReport::getSubmitterId,
                            PmWorkReport::getCreateTime, PmWorkReport::getReportDate)
                    .in(PmWorkReport::getId, reportIds)
                    .orderByDesc(PmWorkReport::getCreateTime);
            reportList = pmWorkReportMapper.selectList(reportQuery);
            LambdaQueryWrapper<PmReceive> statusQuery = Wrappers.lambdaQuery();
            statusQuery.select(PmReceive::getReportId, PmReceive::getReadFlag)
                    .eq(PmReceive::getReceiverId, SecurityUtils.getUserId())
                    .eq(PmReceive::getTenantId, SecurityUtils.getTenantId())
                    .in(PmReceive::getReportId, reportIds);
            List<PmReceive> receiveList = pmReceiveMapper.selectList(statusQuery);
            for (PmWorkReport c : reportList) {
                List<PmWorkContent> contents = new ArrayList<>();
                for (PmWorkContent b : pmWorkContentList) {
                    if (b.getReportId().equals(c.getId())) {
                        contents.add(b);
                    }
                }
                c.setWorkContents(contents);
                if (!receiveList.isEmpty()) {
                    for (PmReceive receive : receiveList) {
                        if (receive.getReportId().equals(c.getId())) {
                            c.setReadFlag(receive.getReadFlag());
                        }
                    }
                }
                userIds.add(c.getSubmitterId());
            }
        }
        List<SysUser> nickNames = nickNames(userIds);
        for (PmWorkReport d : reportList) {
            for (SysUser e : nickNames) {
                if (e.getUserId().equals(d.getSubmitterId())) {
                    d.setCreateBy(e.getNickName());
                }
            }
            List<Long> ids = null;
            if (!d.getCcTo().isEmpty()) {
                ids = Optional.ofNullable(d.getCcTo())
                        .map(ccTo -> Arrays.stream(ccTo.split(","))
                                .map(Long::parseLong)
                                .collect(Collectors.toList()))
                        .orElse(Collections.emptyList());
            }
            List<SysUser> nickNameList = null;
            if (CollectionUtils.isNotEmpty(ids)) {
                nickNameList = nickNames(ids);
                StringBuilder ccToBuilder = new StringBuilder();
                Set<Long> addedUserIds = new HashSet<>();
                for (Long id : ids) {
                    for (SysUser user : nickNameList) {
                        if (user.getUserId().equals(id) && !addedUserIds.contains(id)) {
                            if (ccToBuilder.length() > 0) {
                                ccToBuilder.append(", ");
                            }
                            ccToBuilder.append(user.getNickName());
                            addedUserIds.add(id);
                            break;
                        }
                    }
                }
                String ccTo = ccToBuilder.toString();
                d.setCcTo(ccTo);
            }
        }
        return reportList;
    }

    /**
     * @description: 用户名、账户名查询
     * @author: fu
     * @date: 2024/8/7 17:28
     * @param: [userIds]
     * @return: java.util.List<com.usky.system.domain.SysUser>
     **/
    @Override
    public List<SysUser> nickNames(List<Long> userIds) {
        LambdaQueryWrapper<SysUser> usersQuery = Wrappers.lambdaQuery();
        usersQuery.select(SysUser::getUserId, SysUser::getNickName, SysUser::getUserName, SysUser::getPhonenumber, SysUser::getAvatar,
                        SysUser::getSex, SysUser::getDeptId)
                .eq(SysUser::getTenantId, SecurityUtils.getTenantId())
                .eq(SysUser::getDelFlag, 0)
                .in(SysUser::getUserId, userIds);
        return sysUserMapper.selectList(usersQuery);
    }

    @Override
    public List<SysUser> allNickName() {
        LambdaQueryWrapper<SysUser> usersQuery = Wrappers.lambdaQuery();
        usersQuery.select(SysUser::getUserId, SysUser::getNickName, SysUser::getUserName)
                .eq(SysUser::getTenantId, SecurityUtils.getTenantId())
                .eq(SysUser::getDelFlag, 0);
        return sysUserMapper.selectList(usersQuery);
    }

    @Override
    public CommonPage<PmWorkReport> reportPage(Integer projectAscription, Integer pageNum, Integer pageSize, Integer reportId, String startDate, String endDate, Integer projectId) {
        CommonPage<PmWorkReport> returnPage = new CommonPage<>(new ArrayList<>(), 0, pageSize, pageNum);
        Integer tenantId = SecurityUtils.getTenantId();
        Long userId = SecurityUtils.getUserId();
        String userName = SecurityUtils.getUsername();
        Long deptId = SecurityUtils.getLoginUser().getSysUser().getDeptId();
        List<Integer> headProjects = head();
        if (projectAscription == 1 && headProjects.isEmpty()) {
            return returnPage;
        }
        List<Integer> receiveList2 = receive();
        if (projectAscription == 2 && receiveList2.isEmpty()) {
            return returnPage;
        }
        List<Long> userIds = new ArrayList<>();
        LambdaQueryWrapper<PmWorkContent> contentLambdaQuery = Wrappers.lambdaQuery();
        contentLambdaQuery.select(PmWorkContent::getReportId, PmWorkContent::getWorkContent, PmWorkContent::getWorkTime, PmWorkContent::getProjectName);
        if (reportId != null && reportId != 0) {
            contentLambdaQuery.eq(PmWorkContent::getReportId, reportId);
        } else {
            contentLambdaQuery.eq(PmWorkContent::getTenantId, tenantId).eq(PmWorkContent::getDeptId, deptId);
            if (projectId != null && projectId != 0) {
                contentLambdaQuery.eq(PmWorkContent::getProjectId, projectId);
            }
            if (StringUtils.isNotBlank(startDate) && StringUtils.isNotBlank(endDate)) {
                DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
                LocalDate start = LocalDate.parse(startDate, formatter);
                LocalDate end = LocalDate.parse(endDate, formatter);
                LocalDateTime startDateTime = start.atStartOfDay();
                LocalDateTime endDateTime = end.atTime(23, 59, 59);
                contentLambdaQuery.between(PmWorkContent::getCreateTime, startDateTime, endDateTime);
            }
            switch (projectAscription) {
                case 1:
                    contentLambdaQuery.in(PmWorkContent::getReportId, headProjects);
                    break;
                case 2:
                    contentLambdaQuery.in(PmWorkContent::getReportId, receiveList2);
                    break;
                case 3:
                    contentLambdaQuery.eq(PmWorkContent::getCreateBy, userName);
                    break;
            }
        }
        List<PmWorkContent> pmWorkContentList = pmWorkContentMapper.selectList(contentLambdaQuery);
        Set<Integer> reportIds = pmWorkContentList.stream().map(PmWorkContent::getReportId).collect(Collectors.toSet());
        if (reportIds.isEmpty()) {
            return returnPage;
        }

        LambdaQueryWrapper<PmWorkReport> reportQuery = Wrappers.lambdaQuery();
        reportQuery.select(PmWorkReport::getId, PmWorkReport::getCoordinateWork, PmWorkReport::getTomorrowPlan, PmWorkReport::getCcTo, PmWorkReport::getSubmitterId,
                PmWorkReport::getCreateTime, PmWorkReport::getReportDate);
        if (reportId != null && reportId != 0) {
            reportQuery.eq(PmWorkReport::getId, reportId);
        } else {
            reportQuery.in(PmWorkReport::getId, reportIds)
                    .orderByDesc(PmWorkReport::getCreateTime);
        }
        IPage<PmWorkReport> reportPage = pmWorkReportMapper.selectPage(new Page<>(pageNum, pageSize), reportQuery);
        if (reportPage.getRecords() == null || reportPage.getRecords().isEmpty()) {
            return returnPage;
        }
        List<PmWorkReport> reportList = reportPage.getRecords();

        LambdaQueryWrapper<PmReceive> statusQuery = Wrappers.lambdaQuery();
        statusQuery.select(PmReceive::getReportId, PmReceive::getReadFlag)
                .eq(PmReceive::getReceiverId, userId)
                .eq(PmReceive::getTenantId, tenantId);
        if (reportId != null && reportId != 0) {
            statusQuery.eq(PmReceive::getReportId, reportId);
        } else {
            statusQuery.in(PmReceive::getReportId, reportIds);
        }
        List<PmReceive> receiveList = pmReceiveMapper.selectList(statusQuery);
        Map<Integer, Integer> reportReadFlags = receiveList.stream()
                .collect(Collectors.toMap(PmReceive::getReportId, PmReceive::getReadFlag));
        reportList.forEach(report -> {
            List<PmWorkContent> contents = pmWorkContentList.stream()
                    .filter(content -> content.getReportId().equals(report.getId()))
                    .collect(Collectors.toList());
            report.setWorkContents(contents);
            userIds.add(report.getSubmitterId());
            if (reportReadFlags.containsKey(report.getId())) {
                report.setReadFlag(reportReadFlags.get(report.getId()));
            }
        });

        if (!userIds.isEmpty()) {
            List<SysUser> nickNames = nickNames(userIds);
            Map<Long, String> userNicknameMap = nickNames.stream()
                    .collect(Collectors.toMap(SysUser::getUserId, SysUser::getNickName));
            reportList.forEach(report -> {
                report.setCreateBy(userNicknameMap.get(report.getSubmitterId()));
            });
        }

        // 已读未读查询
        List<Integer> reportIdList = new ArrayList<>(reportIds);
        List<PmReceive> receives = receives(reportIdList);

        List<Long> uId = new ArrayList<>();
        for (PmReceive pmReceive : receives) {
            Long receiverId = pmReceive.getReceiverId();
            uId.add(receiverId);
        }

        List<Long> userIdList = new ArrayList<>();
        for (PmWorkReport report : reportList) {
            userIdList.add(report.getSubmitterId());
        }
        List<SysUser> sysUsers = nickNames(userIdList);

        Map<Integer, List<PmReceive>> reportReceivesMap = receives.stream()
                .collect(Collectors.groupingBy(PmReceive::getReportId));

        if (sysUsers != null) {
            for (PmWorkReport report : reportList) {
                List<Long> readAlready = new ArrayList<>();
                List<Long> readNotAlready = new ArrayList<>();
                int readCount = 0;
                int unreadCount = 0;
                List<PmReceive> reportReceives = reportReceivesMap.getOrDefault(report.getId(), Collections.emptyList());
                for (PmReceive pmReceive : reportReceives) {
                    if (pmReceive.getReadFlag() == 1) {
                        readCount++;
                        readAlready.add(pmReceive.getReceiverId());
                    } else {
                        unreadCount++;
                        readNotAlready.add(pmReceive.getReceiverId());
                    }
                }

                PmReportReadersVO readUnreadVO = new PmReportReadersVO(readAlready, readNotAlready, readCount, unreadCount);
                report.setPmReportReaders(readUnreadVO);

                // 头像
                for (SysUser sysUser : sysUsers) {
                    if (sysUser.getUserId().equals(report.getSubmitterId())) {
                        report.setAvatar(sysUser.getAvatar());
                        break;
                    }
                }
            }
        }

        return new CommonPage<>(reportList, reportPage.getTotal(), pageSize, pageNum);
    }

    /**
     * @description: 获取报告接收人
     * @author: fu
     * @date: 2024/8/8 19:22
     * @param: [report]
     * @return: java.util.List<com.usky.iot.domain.PmReceive>
     **/
    private List<PmReceive> receives(List<Integer> reports) {
        LambdaQueryWrapper<PmReceive> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.select(PmReceive::getReportId, PmReceive::getReadFlag, PmReceive::getReceiverId)
                .in(PmReceive::getReportId, reports);
        return pmReceiveMapper.selectList(queryWrapper);
    }

    /**
     * @description: 获取报告列表
     * @author: fu
     * @date: 2024/8/8 19:22
     * @param: [startDate, endDate]
     * @return: java.util.List<java.lang.Integer>
     **/
    private List<Integer> reports(LocalDate startDate, LocalDate endDate) {
        LambdaQueryWrapper<PmWorkReport> reportsQuery = Wrappers.lambdaQuery();
        reportsQuery.select(PmWorkReport::getId)
                .between(PmWorkReport::getReportDate, startDate, endDate)
                .groupBy(PmWorkReport::getId);
        List<PmWorkReport> pmWorkReportList = pmWorkReportMapper.selectList(reportsQuery);
        return pmWorkReportList.stream()
                .map(PmWorkReport::getId)
                .collect(Collectors.toList());
    }

    private List<PmProject> projects(Integer projectId) {
        Integer tenantId = SecurityUtils.getTenantId();
        Long deptId = SecurityUtils.getLoginUser().getSysUser().getDeptId();
        LambdaQueryWrapper<PmProject> projectsQuery = Wrappers.lambdaQuery();
        projectsQuery.select(PmProject::getId, PmProject::getProjectName);
        if (projectId != null && projectId != 0) {
            projectsQuery.eq(PmProject::getId, projectId);
            return pmProjectMapper.selectList(projectsQuery);
        }
        projectsQuery.eq(PmProject::getTenantId, tenantId).eq(PmProject::getDeptId, deptId).eq(PmProject::getDelFlag, 0);
        return pmProjectMapper.selectList(projectsQuery);
    }

    private List<SysUser> userNameList(Long userId) {
        Integer tenantId = SecurityUtils.getTenantId();
        Long deptId = SecurityUtils.getLoginUser().getSysUser().getDeptId();
        LambdaQueryWrapper<SysUser> userNameQuery = Wrappers.lambdaQuery();
        userNameQuery.select(SysUser::getUserId, SysUser::getNickName, SysUser::getUserName, SysUser::getPhonenumber, SysUser::getAvatar,
                        SysUser::getSex, SysUser::getDeptId)
                .eq(SysUser::getDelFlag, 0)
                .eq(SysUser::getStatus, 0);
        if (userId != null && userId != 0) {
            userNameQuery.eq(SysUser::getUserId, userId);
            return sysUserMapper.selectList(userNameQuery);
        }
        userNameQuery.eq(SysUser::getTenantId, tenantId)
                .eq(SysUser::getDeptId, deptId);
        return sysUserMapper.selectList(userNameQuery);
    }

    //@DataScope//数据权限注解
    private List<PmWorkContent> workTimeCount(List<Long> users, List<Integer> project, String startDate, String endDate) {
        Integer tenantId = SecurityUtils.getTenantId();
        Long deptId = SecurityUtils.getLoginUser().getSysUser().getDeptId();
        LambdaQueryWrapper<PmWorkContent> workTimeQuery = Wrappers.lambdaQuery();
        workTimeQuery.select(PmWorkContent::getSubmitterId, PmWorkContent::getReportId, PmWorkContent::getProjectId, PmWorkContent::getProjectName,
                PmWorkContent::getWorkTime, PmWorkContent::getDeptId, PmWorkContent::getTenantId);
        if (!users.isEmpty()) {
            workTimeQuery.in(PmWorkContent::getSubmitterId, users);
        }
        if (!project.isEmpty()) {
            workTimeQuery.in(PmWorkContent::getProjectId, project);
        }
        if (StringUtils.isNotBlank(startDate) && StringUtils.isNotBlank(endDate)) {
            LocalDate start = LocalDate.parse(startDate);
            LocalDate end = LocalDate.parse(endDate);
            List<Integer> reports = reports(start, end);
            workTimeQuery.in(PmWorkContent::getReportId, reports);
        }
        workTimeQuery.eq(PmWorkContent::getTenantId, tenantId).eq(PmWorkContent::getDeptId, deptId);
        // workTimeQuery.apply(DataScopeContextHolder.getDataScopeSql());// 数据权限设置
        List<PmWorkContent> pmWorkContentList = pmWorkContentMapper.selectList(workTimeQuery);

        Map<String, PmWorkContent> workTimeMap = new HashMap<>();
        for (PmWorkContent content : pmWorkContentList) {
            String key = content.getSubmitterId() + "_" + content.getProjectId();
            PmWorkContent existingContent = workTimeMap.get(key);
            if (existingContent == null) {
                workTimeMap.put(key, content);
            } else {
                existingContent.setWorkTime(existingContent.getWorkTime().add(content.getWorkTime()));
            }
        }

        return new ArrayList<>(workTimeMap.values());
    }


    @Override
    public List<Object> workHourStatistic(Long userId, Integer projectId, Integer filter, String startDate, String endDate, Integer workerOrProject) {
        List<ProjectWorkTimeVO> projectWorkTime = new ArrayList<>();
        List<UserWorkTimeVO> userWorkTimeVO = new ArrayList<>();
        Integer tenantId = SecurityUtils.getTenantId();
        List<Object> returnList = new ArrayList<>();
        if (StringUtils.isBlank(startDate) && StringUtils.isBlank(endDate)) {
            throw new BusinessException("请设置查询时间范围！");
        }

        List<PmProject> pmProjects = projects(projectId);
        List<String> pmProjectName = new ArrayList<>();
        List<Integer> pmProjectId = new ArrayList<>();
        for (PmProject project : pmProjects) {
            pmProjectName.add(project.getProjectName());
            pmProjectId.add(project.getId());
        }

        List<SysUser> users = userNameList(userId);
        List<Long> userIds = new ArrayList<>();
        List<String> nickName = new ArrayList<>();
        for (SysUser user : users) {
            userIds.add(user.getUserId());
            nickName.add(user.getNickName());
        }

        // 统计userId每个项目所需工时
        List<WorkHoursStatisticsVO> userAndProject = new ArrayList<>();
        List<PmWorkContent> pmWorkContentList = workTimeCount(userIds, pmProjectId, startDate, endDate);
        for (PmWorkContent content : pmWorkContentList) {
            WorkHoursStatisticsVO workHoursStatisticsVO = new WorkHoursStatisticsVO();
            Long submitterId = content.getSubmitterId();
            Integer projectId1 = content.getProjectId();
            BigDecimal workTime = content.getWorkTime();
            workHoursStatisticsVO.setWorkTime(workTime);
            for (SysUser user : users) {
                Long userId1 = user.getUserId();
                String nickName1 = user.getNickName();
                if (userId1.equals(submitterId)) {
                    workHoursStatisticsVO.setNickName(nickName1);
                }
            }
            for (PmProject project : pmProjects) {
                Integer projectId2 = project.getId();
                String projectName = project.getProjectName();
                if (projectId2 == projectId1) {
                    workHoursStatisticsVO.setProjectName(projectName);
                }
            }
            userAndProject.add(workHoursStatisticsVO);
        }
        List<WorkHoursStatisticsVO> userAndProjectD = userAndProject;

        // userAndProject = baseMapper.workHoursStatistics(start, end, userId, tenantId);
        // Long deptId = SecurityUtils.getLoginUser().getSysUser().getDeptId();
        // List<WorkHoursStatisticsVO> userAndProjectD = baseMapper.statisticsDept(start, end, userId, tenantId, deptId);
        if (workerOrProject != 1) {
            returnList.add(nickName);
            // 重组返回数据结构
            for (String name : pmProjectName) {
                ProjectWorkTimeVO projectWorkTimeVO = new ProjectWorkTimeVO();
                projectWorkTimeVO.setProjectOrPerson(name);
                List<BigDecimal> workTime = new ArrayList<>();
                boolean hasNonZeroWorkTime = false;
                for (String currentNickName : nickName) {
                    boolean foundWorkTime = false;
                    for (WorkHoursStatisticsVO workHoursStatisticsVO : userAndProject) {
                        String userNickName = workHoursStatisticsVO.getNickName();
                        String projectName = workHoursStatisticsVO.getProjectName();
                        BigDecimal userProjectWorkTime = workHoursStatisticsVO.getWorkTime();
                        if (projectName == name && userNickName == currentNickName) {
                            workTime.add(userProjectWorkTime);
                            if (userProjectWorkTime.compareTo(BigDecimal.ZERO) != 0) {
                                hasNonZeroWorkTime = true;
                            }
                            foundWorkTime = true;
                            break;
                        }
                    }
                    if (!foundWorkTime) {
                        workTime.add(BigDecimal.ZERO);
                    }
                }
                if (filter != 2 || hasNonZeroWorkTime) {
                    projectWorkTimeVO.setWorkTime(workTime);
                    projectWorkTime.add(projectWorkTimeVO);
                }
            }
            returnList.add(projectWorkTime);
        } else {
            returnList.add(pmProjectName);
            // 重组返回数据结构
            for (String uName : nickName) {
                String name = uName;
                UserWorkTimeVO userWorkTimeVO2 = new UserWorkTimeVO();
                userWorkTimeVO2.setProjectOrPerson(name);
                List<BigDecimal> workTime = new ArrayList<>();
                boolean hasNonZeroWorkTime = false;
                for (String currentNickName : pmProjectName) {
                    boolean foundWorkTime = false;
                    for (WorkHoursStatisticsVO workHoursStatisticsVO : userAndProjectD) {
                        String userNickName = workHoursStatisticsVO.getNickName();
                        String projectName = workHoursStatisticsVO.getProjectName();
                        BigDecimal userProjectWorkTime = workHoursStatisticsVO.getWorkTime();
                        if (userNickName == name && projectName == currentNickName) {
                            workTime.add(userProjectWorkTime);
                            if (userProjectWorkTime.compareTo(BigDecimal.ZERO) != 0) {
                                hasNonZeroWorkTime = true;
                            }
                            foundWorkTime = true;
                            break;
                        }
                    }
                    if (!foundWorkTime) {
                        workTime.add(BigDecimal.ZERO);
                    }
                }
                if (filter != 2 || hasNonZeroWorkTime) {
                    userWorkTimeVO2.setWorkTime(workTime);
                    userWorkTimeVO.add(userWorkTimeVO2);
                }
            }
            returnList.add(userWorkTimeVO);
        }
        return returnList;
    }

    @Override
    public List<WorkTimeExportVO> workHourStatisticExport(Long userId, Integer projectId, String startDate, String endDate) {
        Integer tenantId = SecurityUtils.getTenantId();
        LocalDate start = null;
        LocalDate end = null;
        if (StringUtils.isNotBlank(startDate) && StringUtils.isNotBlank(endDate)) {
            start = LocalDate.parse(startDate);
            end = LocalDate.parse(endDate);
        }
//        else {
//            throw new BusinessException("导出数据过多请输入正确时间进行筛选");
//        }
        return pmWorkContentMapper.workHourStatisticExport(userId, projectId, start, end, tenantId);
    }

}
